@@ -2,7 +2,7 @@ package WebService::ILS;
 
 use Modern::Perl;
 
-our $VERSION = "0.06";
+our $VERSION = "0.07";
 
 =encoding utf-8
 
@@ -75,25 +75,39 @@ Item record is returned by many methods, so we specify it here.
 
 =item C<id>
 
+=item C<isbn>
+
 =item C<title>
 
 =item C<subtitle>
 
-=item C<subtitle>
+=item C<description>
 
 =item C<author>
 
 =item C<publisher>
 
-=item C<rating>     => user ratings metrics
+=item C<publication_date>
 
-=item C<popularity> => checkout metrics
+=item C<language>
 
-=item C<subjects>   => subject categories (tags, facets...)
+=item C<rating>         => user ratings metrics
 
-=item C<media>      => book, e-book, video, audio etc
+=item C<popularity>     => checkout metrics
 
-=item C<formats>    => an arrayref of available formats
+=item C<subjects>       => subject categories (tags)
+
+=item C<facets>         => a hashref of facet => [values]
+
+=item C<media>          => book, e-book, video, audio etc
+
+=item C<formats>        => an arrayref of available formats
+
+=item C<images>         => a hashref of size => url
+
+=item C<encryption_key> => for decryption purposes
+
+=item C<drm>            => subject to drm
 
 =back
 
@@ -300,7 +314,7 @@ sub search {
 }
 
 # relevancy availability available_date title author popularity rating price publisher publication_date
-sub _build_sort_string {
+sub _parse_sort_string {
     my $self = shift;
     my $sort = shift or croak "No sort options";
     my $xlate_table = shift || {};
@@ -321,7 +335,8 @@ sub _build_sort_string {
         $_ = join '', map ucfirst, split /(?<=[A-Za-z])_(?=[A-Za-z])|\b/ if $camelise;
         $_ = "$_:$d" if $d;
     }
-    return join ",", @$sort;
+
+    return $sort;
 }
 
 sub item_availability {
@@ -345,7 +360,7 @@ An example:
     eval { $ils->auth_by_user_id( $user_id, $password ) };
     if ($@) { some_error_handling(); return; }
     $session{ils_access_token} = $ils->access_token;
-    $session{ils_access_token_type} = $ils->access_token;
+    $session{ils_access_token_type} = $ils->access_token_type;
     ...
     Somewhere else in your app:
     my $ils = WebService::ILS::Provider({
@@ -396,7 +411,7 @@ An example:
     eval { $ils->auth_by_token( $auth_token ) };
     if ($@) { some_error_handling(); return; }
     $session{ils_access_token} = $ils->access_token;
-    $session{ils_access_token_type} = $ils->access_token;
+    $session{ils_access_token_type} = $ils->access_token_type;
     ...
     Somewhere else in your app:
     passing access token to the constructor as above
@@ -481,6 +496,11 @@ item records will have:
 
 =item C<expires>           => date (time) checkout expires
 
+=item C<url>               => download/stream url
+
+=item C<files>             => an arrayref of downloadable file details
+                              title, url, size
+
 =back
 
 =head2 checkout ($item_id)
@@ -527,7 +547,7 @@ native methods.
 
 =cut
 
-use constant ERROR_ACCESS_TOKEN => "Error: Invalid Access Token";
+use constant ERROR_ACCESS_TOKEN => "Error: Authorization Failed";
 use constant ERROR_NOT_AUTHENTICATED => "Error: User Not Authenticated";
 
 sub invalid_response_exception_string {
@@ -613,7 +633,7 @@ sub _request_access_token {
     my $request = shift or croak "No request";
 
     $request->header(
-        Authorization => "Basic " . MIME::Base64::encode( $self->_access_auth_string )
+        Authorization => "Basic " . $self->_access_auth_string
     );
 
     my $response = $self->user_agent->request( $request );
@@ -623,9 +643,10 @@ sub _request_access_token {
         sub {
             my ($data) = @_;
 
-            my $token = $data->{access_token} or die "No access token\n";
+            my ($token, $token_type) = $self->_extract_token_from_response($data);
+            $token or die "No access token\n";
             $self->access_token($token);
-            $self->access_token_type($data->{token_type} || 'Bearer');
+            $self->access_token_type($token_type || 'Bearer');
             return $data;
         },
         sub {
@@ -638,7 +659,14 @@ sub _request_access_token {
 
 sub _access_auth_string {
     my $self = shift;
-    return join ":", $self->client_id, $self->client_secret;
+    return MIME::Base64::encode( join(":", $self->client_id, $self->client_secret) );
+}
+
+sub _extract_token_from_response {
+    my $self = shift;
+    my $data = shift;
+
+    return ($data->{access_token}, $data->{token_type});
 }
 
 sub _request_with_token {
@@ -703,7 +731,7 @@ sub process_json_error_response {
             or die $self->invalid_response_exception_string($response);
 
         my $data = eval { from_json( $content ) };
-        die "$@\nResponse:\n$content" if $@;
+        die $content || $self->invalid_response_exception_string($response) if $@;
 
         if ($error_callback) {
             return $error_callback->($data);
